@@ -1,8 +1,10 @@
 use ethabi::Token;
+use std::collections::HashMap;
 use std::time::{Duration, Instant};
 use serde_json::{Value as JSONValue, json};
 
 use crate::abi::ABI;
+use crate::config::Config;
 use crate::transaction::get_sign_transaction_data;
 use crate::account::{Account, create_account_from_pem};
 use crate::web3::{fetcher_trait::FetcherTrait, service_error::ServiceError};
@@ -25,88 +27,79 @@ pub struct CallResponse {
 }
 
 pub struct Service {
-    group_id: u32,
-    chain_id: u32,
-    sm_crypto: bool,
-    timeout_seconds: i64,
+    pub config: Config,
     account: Account,
     fetcher: Box<dyn FetcherTrait + Send + Sync>,
 }
 
 impl Service {
-    pub fn new(
-        group_id: u32,
-        chain_id: u32,
-        timeout_seconds: i64,
-        account_pem_file_path: &str,
-        sm_crypto: bool,
-        fetcher: Box<dyn FetcherTrait + Send + Sync>,
-    ) -> Result<Service, ServiceError> {
+    fn get_abi(&self, contract_name: &str) -> Result<ABI, ServiceError> {
+        Ok(ABI::new(&self.config.contract, contract_name, self.config.sm_crypto)?)
+    }
+
+    pub fn new(config: &Config, fetcher: Box<dyn FetcherTrait + Send + Sync>) -> Result<Service, ServiceError> {
         Ok(
             Service {
-                group_id,
-                chain_id,
-                sm_crypto,
-                timeout_seconds,
                 fetcher,
-                account: create_account_from_pem(account_pem_file_path, sm_crypto)?,
+                config: config.clone(),
+                account: create_account_from_pem(&config.account, config.sm_crypto)?,
             }
         )
     }
 
     pub async fn get_client_version(&self)  -> Result<JSONValue, ServiceError> {
-        let params = generate_request_params("getClientVersion", &json!([self.group_id]));
+        let params = generate_request_params("getClientVersion", &json!([self.config.group_id]));
         Ok(self.fetcher.fetch(&params).await?)
     }
 
     pub async fn get_block_number(&self) -> Result<String, ServiceError> {
-        let params = generate_request_params("getBlockNumber", &json!([self.group_id]));
+        let params = generate_request_params("getBlockNumber", &json!([self.config.group_id]));
         let response = self.fetcher.fetch(&params).await?;
         Ok(parse_json_string(&response))
     }
 
     pub async fn get_pbft_view(&self) -> Result<String, ServiceError> {
-        let params = generate_request_params("getPbftView", &json!([self.group_id]));
+        let params = generate_request_params("getPbftView", &json!([self.config.group_id]));
         let response = self.fetcher.fetch(&params).await?;
         Ok(parse_json_string(&response))
     }
 
     pub async fn get_sealer_list(&self) -> Result<Vec<String>, ServiceError> {
-        let params = generate_request_params("getSealerList", &json!([self.group_id]));
+        let params = generate_request_params("getSealerList", &json!([self.config.group_id]));
         let response = self.fetcher.fetch(&params).await?;
         Ok(parse_json_string_array(&response))
     }
 
     pub async fn get_observer_list(&self) -> Result<Vec<String>, ServiceError> {
-        let params = generate_request_params("getObserverList", &json!([self.group_id]));
+        let params = generate_request_params("getObserverList", &json!([self.config.group_id]));
         let response = self.fetcher.fetch(&params).await?;
         Ok(parse_json_string_array(&response))
     }
 
     pub async fn get_consensus_status(&self) -> Result<JSONValue, ServiceError> {
-        let params = generate_request_params("getConsensusStatus", &json!([self.group_id]));
+        let params = generate_request_params("getConsensusStatus", &json!([self.config.group_id]));
         Ok(self.fetcher.fetch(&params).await?)
     }
 
     pub async fn get_sync_status(&self) -> Result<JSONValue, ServiceError> {
-        let params = generate_request_params("getSyncStatus", &json!([self.group_id]));
+        let params = generate_request_params("getSyncStatus", &json!([self.config.group_id]));
         Ok(self.fetcher.fetch(&params).await?)
     }
 
     pub async fn get_peers(&self) -> Result<Vec<JSONValue>, ServiceError> {
-        let params = generate_request_params("getPeers", &json!([self.group_id]));
+        let params = generate_request_params("getPeers", &json!([self.config.group_id]));
         let response = self.fetcher.fetch(&params).await?;
         Ok(response.as_array().unwrap().clone())
     }
 
     pub async fn get_group_peers(&self) -> Result<Vec<String>, ServiceError> {
-        let params = generate_request_params("getGroupPeers", &json!([self.group_id]));
+        let params = generate_request_params("getGroupPeers", &json!([self.config.group_id]));
         let response = self.fetcher.fetch(&params).await?;
         Ok(parse_json_string_array(&response))
     }
 
     pub async fn get_node_id_list(&self) -> Result<Vec<String>, ServiceError> {
-        let params = generate_request_params("getNodeIDList", &json!([self.group_id]));
+        let params = generate_request_params("getNodeIDList", &json!([self.config.group_id]));
         let response = self.fetcher.fetch(&params).await?;
         Ok(parse_json_string_array(&response))
     }
@@ -120,7 +113,7 @@ impl Service {
     pub async fn get_block_by_hash(&self, block_hash: &str, include_transactions: bool) -> Result<JSONValue, ServiceError> {
         let params = generate_request_params(
             "getBlockByHash",
-            &json!([self.group_id, block_hash, include_transactions]),
+            &json!([self.config.group_id, block_hash, include_transactions]),
         );
         Ok(self.fetcher.fetch(&params).await?)
     }
@@ -128,7 +121,7 @@ impl Service {
     pub async fn get_block_by_number(&self, block_number: &str, include_transactions: bool)-> Result<JSONValue, ServiceError> {
         let params = generate_request_params(
             "getBlockByNumber",
-            &json!([self.group_id, block_number, include_transactions]),
+            &json!([self.config.group_id, block_number, include_transactions]),
         );
         Ok(self.fetcher.fetch(&params).await?)
     }
@@ -136,7 +129,7 @@ impl Service {
     pub async fn get_block_header_by_hash(&self, block_hash: &str, include_transactions: bool) -> Result<JSONValue, ServiceError> {
         let params = generate_request_params(
             "getBlockHeaderByHash",
-            &json!([self.group_id, block_hash, include_transactions]),
+            &json!([self.config.group_id, block_hash, include_transactions]),
         );
         Ok(self.fetcher.fetch(&params).await?)
     }
@@ -144,7 +137,7 @@ impl Service {
     pub async fn get_block_header_by_number(&self, block_number: &str, include_transactions: bool) -> Result<JSONValue, ServiceError> {
         let params = generate_request_params(
             "getBlockHeaderByNumber",
-            &json!([self.group_id, block_number, include_transactions]),
+            &json!([self.config.group_id, block_number, include_transactions]),
         );
         Ok(self.fetcher.fetch(&params).await?)
     }
@@ -152,7 +145,7 @@ impl Service {
     pub async fn get_block_hash_by_number(&self, block_number: &str) -> Result<String, ServiceError> {
         let params = generate_request_params(
             "getBlockHashByNumber",
-            &json!([self.group_id, block_number]),
+            &json!([self.config.group_id, block_number]),
         );
         let response = self.fetcher.fetch(&params).await?;
         Ok(parse_json_string(&response))
@@ -161,7 +154,7 @@ impl Service {
     pub async fn get_transaction_by_hash(&self, transaction_hash: &str) -> Result<JSONValue, ServiceError> {
         let params = generate_request_params(
             "getTransactionByHash",
-            &json!([self.group_id, transaction_hash]),
+            &json!([self.config.group_id, transaction_hash]),
         );
         Ok(self.fetcher.fetch(&params).await?)
     }
@@ -169,7 +162,7 @@ impl Service {
     pub async fn get_transaction_by_block_hash_and_index(&self, block_hash: &str, transaction_index: &str) -> Result<JSONValue, ServiceError> {
         let params = generate_request_params(
             "getTransactionByBlockHashAndIndex",
-            &json!([self.group_id, block_hash, transaction_index]),
+            &json!([self.config.group_id, block_hash, transaction_index]),
         );
         Ok(self.fetcher.fetch(&params).await?)
     }
@@ -177,7 +170,7 @@ impl Service {
     pub async fn get_transaction_by_block_number_and_index(&self, block_number: &str, transaction_index: &str) -> Result<JSONValue, ServiceError> {
         let params = generate_request_params(
             "getTransactionByBlockNumberAndIndex",
-            &json!([self.group_id, block_number, transaction_index]),
+            &json!([self.config.group_id, block_number, transaction_index]),
         );
         Ok(self.fetcher.fetch(&params).await?)
     }
@@ -185,7 +178,7 @@ impl Service {
     pub async fn get_transaction_receipt(&self, transaction_hash: &str) -> Result<JSONValue, ServiceError> {
         let params = generate_request_params(
             "getTransactionReceipt",
-            &json!([self.group_id, transaction_hash]),
+            &json!([self.config.group_id, transaction_hash]),
         );
         Ok(self.fetcher.fetch(&params).await?)
     }
@@ -193,7 +186,7 @@ impl Service {
     pub async fn get_pending_transactions(&self) -> Result<JSONValue, ServiceError> {
         let params = generate_request_params(
             "getPendingTransactions",
-            &json!([self.group_id]),
+            &json!([self.config.group_id]),
         );
         Ok(self.fetcher.fetch(&params).await?)
     }
@@ -201,7 +194,7 @@ impl Service {
     pub async fn get_pending_tx_size(&self) -> Result<String, ServiceError> {
         let params = generate_request_params(
             "getPendingTxSize",
-            &json!([self.group_id]),
+            &json!([self.config.group_id]),
         );
         let response = self.fetcher.fetch(&params).await?;
         Ok(parse_json_string(&response))
@@ -210,7 +203,7 @@ impl Service {
     pub async fn get_code(&self, address: &str) -> Result<String, ServiceError> {
         let params = generate_request_params(
             "getCode",
-            &json!([self.group_id, address]),
+            &json!([self.config.group_id, address]),
         );
         let response = self.fetcher.fetch(&params).await?;
         Ok(parse_json_string(&response))
@@ -219,7 +212,7 @@ impl Service {
     pub async fn get_total_transaction_count(&self) -> Result<JSONValue, ServiceError> {
         let params = generate_request_params(
             "getTotalTransactionCount",
-            &json!([self.group_id]),
+            &json!([self.config.group_id]),
         );
         Ok(self.fetcher.fetch(&params).await?)
     }
@@ -227,7 +220,7 @@ impl Service {
     pub async fn get_system_config_by_key(&self, key: &str) -> Result<String, ServiceError> {
         let params = generate_request_params(
             "getSystemConfigByKey",
-            &json!([self.group_id, key]),
+            &json!([self.config.group_id, key]),
         );
         let response = self.fetcher.fetch(&params).await?;
         Ok(parse_json_string(&response))
@@ -235,12 +228,12 @@ impl Service {
 
     pub async fn call(
         &self,
-        abi_path: &str,
+        contract_name: &str,
         to_address: &str,
         function_name: &str,
         tokens: &Vec<Token>,
     ) -> Result<CallResponse, ServiceError> {
-        let abi = ABI::new(abi_path, self.sm_crypto)?;
+        let abi = self.get_abi(contract_name)?;
         let transaction_data = abi.encode_function_input(function_name, tokens)?;
         let params = json!({
             "from": format!("0x{}", hex::encode(&self.account.address)),
@@ -249,7 +242,7 @@ impl Service {
             "data": format!("0x{}", hex::encode(&transaction_data)),
         });
         let response = self.fetcher.fetch(
-            &generate_request_params("call", &json!([self.group_id, params]))
+            &generate_request_params("call", &json!([self.config.group_id, params]))
         ).await?;
         Ok(CallResponse {
             status: parse_json_string(&response["status"]),
@@ -261,70 +254,70 @@ impl Service {
     async fn send_transaction(
         &self,
         method: &str,
-        abi_path: &str,
+        contract_name: &str,
         to_address: &str,
         function_name: &str,
         tokens: &Vec<Token>,
     ) -> Result<String, ServiceError> {
         let block_number = convert_hex_str_to_u32(&self.get_block_number().await?);
-        let abi = ABI::new(abi_path, self.sm_crypto)?;
+        let abi = self.get_abi(contract_name)?;
         let data = abi.encode_function_input(function_name, tokens)?;
         let transaction_data = get_sign_transaction_data(
             &self.account,
-            self.group_id,
-            self.chain_id,
+            self.config.group_id,
+            self.config.chain_id,
             block_number + 500,
             to_address,
             &data,
-            self.sm_crypto,
+            self.config.sm_crypto,
         )?;
         let params = generate_request_params(
             method,
-            &json!([self.group_id, format!("0x{}", hex::encode(&transaction_data))]),
+            &json!([self.config.group_id, format!("0x{}", hex::encode(&transaction_data))]),
         );
         Ok(parse_json_string(&self.fetcher.fetch(&params).await?))
     }
 
     pub async fn send_raw_transaction(
         &self,
-        abi_path: &str,
+        contract_name: &str,
         to_address: &str,
         function_name: &str,
         tokens: &Vec<Token>,
     ) -> Result<String, ServiceError> {
-        Ok(self.send_transaction("sendRawTransaction", abi_path, to_address, function_name, tokens).await?)
+        Ok(self.send_transaction("sendRawTransaction", contract_name, to_address, function_name, tokens).await?)
     }
 
     pub async fn send_raw_transaction_and_get_proof(
         &self,
-        abi_path: &str,
+        contract_name: &str,
         to_address: &str,
         function_name: &str,
         tokens: &Vec<Token>,
     ) -> Result<String, ServiceError> {
-        Ok(self.send_transaction("sendRawTransactionAndGetProof", abi_path, to_address, function_name, tokens).await?)
+        Ok(self.send_transaction("sendRawTransactionAndGetProof", contract_name, to_address, function_name, tokens).await?)
     }
 
-    pub async fn deploy(&self, abi_bin_path: &str, abi_path: &str, tokens: &Vec<Token>) -> Result<JSONValue, ServiceError> {
+    pub async fn deploy(&self, contract_name: &str, tokens: &Vec<Token>) -> Result<JSONValue, ServiceError> {
         let block_number = convert_hex_str_to_u32(&self.get_block_number().await?);
-        let abi = ABI::new(abi_path, self.sm_crypto)?;
-        let data = abi.encode_constructor_input(abi_bin_path, tokens)?;
+        let abi = self.get_abi(contract_name)?;
+        let data = abi.encode_constructor_input(tokens)?;
         let transaction_data = get_sign_transaction_data(
             &self.account,
-            self.group_id,
-            self.chain_id,
+            self.config.group_id,
+            self.config.chain_id,
             block_number + 500,
             "",
             &data,
-            self.sm_crypto,
+            self.config.sm_crypto,
         )?;
         let params = generate_request_params(
             "sendRawTransactionAndGetProof",
-            &json!([self.group_id, format!("0x{}", hex::encode(&transaction_data))]),
+            &json!([self.config.group_id, format!("0x{}", hex::encode(&transaction_data))]),
         );
         let transaction_hash = parse_json_string(&self.fetcher.fetch(&params).await?);
         let start = Instant::now();
-        let timeout_milliseconds = (1000 * self.timeout_seconds) as u128;
+        let timeout_milliseconds = (1000 * self.config.timeout_seconds) as u128;
         while Instant::now().duration_since(start).as_millis() < timeout_milliseconds {
             let transaction_receipt: JSONValue = self.get_transaction_receipt(&transaction_hash).await?;
             if transaction_receipt.is_null() {
@@ -347,10 +340,22 @@ impl Service {
         })
     }
 
+    ///
+    /// link_libraries 中的键为要链接的 library 的名称，其值为要链接的 library 的地址
+    ///
+    pub async fn compile(
+        &self,
+        contract_name: &str,
+        link_libraries: &Option<HashMap<String, String>>,
+    ) -> Result<(), ServiceError> {
+        let mut abi = self.get_abi(contract_name)?;
+        Ok(abi.compile(link_libraries)?)
+    }
+
     pub async fn get_transaction_by_hash_with_proof(&self, transaction_hash: &str) -> Result<JSONValue, ServiceError> {
         let params = generate_request_params(
             "getTransactionByHashWithProof",
-            &json!([self.group_id, transaction_hash]),
+            &json!([self.config.group_id, transaction_hash]),
         );
         Ok(self.fetcher.fetch(&params).await?)
     }
@@ -358,7 +363,7 @@ impl Service {
     pub async fn get_transaction_receipt_by_hash_with_proof(&self, transaction_hash: &str) -> Result<JSONValue, ServiceError> {
         let params = generate_request_params(
             "getTransactionReceiptByHashWithProof",
-            &json!([self.group_id, transaction_hash]),
+            &json!([self.config.group_id, transaction_hash]),
         );
         Ok(self.fetcher.fetch(&params).await?)
     }
@@ -373,14 +378,14 @@ impl Service {
     /// | enable_free_storage | bool | 可选，是否启用 free storage 模式，启用后节点将减少 STORAGE 相关指令的 gas 耗费 |
     ///
     pub async fn generate_group(&self, params: &JSONValue) -> Result<JSONValue, ServiceError> {
-        let request_params = json!([self.group_id, params.clone()]);
+        let request_params = json!([self.config.group_id, params.clone()]);
         Ok(self.fetcher.fetch(&generate_request_params("generateGroup", &request_params)).await?)
     }
 
     pub async fn start_group(&self) -> Result<JSONValue, ServiceError> {
         let params = generate_request_params(
             "startGroup",
-            &json!([self.group_id]),
+            &json!([self.config.group_id]),
         );
         Ok(self.fetcher.fetch(&params).await?)
     }
@@ -388,7 +393,7 @@ impl Service {
     pub async fn stop_group(&self) -> Result<JSONValue, ServiceError> {
         let params = generate_request_params(
             "stopGroup",
-            &json!([self.group_id]),
+            &json!([self.config.group_id]),
         );
         Ok(self.fetcher.fetch(&params).await?)
     }
@@ -396,7 +401,7 @@ impl Service {
     pub async fn remove_group(&self) -> Result<JSONValue, ServiceError> {
         let params = generate_request_params(
             "removeGroup",
-            &json!([self.group_id]),
+            &json!([self.config.group_id]),
         );
         Ok(self.fetcher.fetch(&params).await?)
     }
@@ -404,7 +409,7 @@ impl Service {
     pub async fn recover_group(&self) -> Result<JSONValue, ServiceError> {
         let params = generate_request_params(
             "recoverGroup",
-            &json!([self.group_id]),
+            &json!([self.config.group_id]),
         );
         Ok(self.fetcher.fetch(&params).await?)
     }
@@ -412,7 +417,7 @@ impl Service {
     pub async fn query_group_status(&self) -> Result<JSONValue, ServiceError> {
         let params = generate_request_params(
             "queryGroupStatus",
-            &json!([self.group_id]),
+            &json!([self.config.group_id]),
         );
         Ok(self.fetcher.fetch(&params).await?)
     }
@@ -431,7 +436,7 @@ impl Service {
     ) -> Result<JSONValue, ServiceError> {
         let params = generate_request_params(
             "getBatchReceiptsByBlockNumberAndRange",
-            &json!([self.group_id, block_number, from, count, compress_flag]),
+            &json!([self.config.group_id, block_number, from, count, compress_flag]),
         );
         Ok(self.fetcher.fetch(&params).await?)
     }
@@ -445,7 +450,7 @@ impl Service {
     ) -> Result<JSONValue, ServiceError> {
         let params = generate_request_params(
             "getBatchReceiptsByBlockHashAndRange",
-            &json!([self.group_id, block_hash, from, count, compress_flag]),
+            &json!([self.config.group_id, block_hash, from, count, compress_flag]),
         );
         Ok(self.fetcher.fetch(&params).await?)
     }
